package com.sztzjy.resource_center.controller.api;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageInfo;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.*;
import com.sztzjy.resource_center.entity.dto.TrainingDto;
import com.sztzjy.resource_center.mapper.SysCaseQuestionMapper;
import com.sztzjy.resource_center.mapper.SysCaseQuestionStepMapper;
import com.sztzjy.resource_center.mapper.SysOneCatalogMapper;
import com.sztzjy.resource_center.mapper.SysTopicAndCourseMapper;
import com.sztzjy.resource_center.util.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@RestController
@Api(tags = "案例题方面API")
@RequestMapping("api/tea/CaseApi")
public class CaseApi {

    @Autowired
    private SysCaseQuestionStepMapper caseQuestionStepMapper;
    @Autowired
    private SysCaseQuestionMapper caseQuestionMapper;
    @Autowired
    private SysTopicAndCourseMapper topicAndCourseMapper;
    @Autowired
    private SysOneCatalogMapper sysOneCatalogMapper;

    //案例题新增 SysCaseQuestion
    @AnonymousAccess
    @ApiOperation("案例题新增")
    @PostMapping("insertCase")
    public Boolean insertCase(@ApiParam("目前二级三级ID可以为空，来源和题型要传,上架状态传true") @RequestBody SysCaseQuestion sysCaseQuestion) {
        SysOneCatalog sysOneCatalogs1 = getSysOneCatalogs(sysCaseQuestion.getOneName());
        String oneId = sysOneCatalogs1.getOneId();

        if (("3").equals(sysCaseQuestion.getType())) {
            return false;
        }
        if (StringUtils.isBlank(sysCaseQuestion.getOneName())) {  //todo 目前二级三级ID可以为空
            return false;
        }
        if (StringUtils.isBlank(sysCaseQuestion.getContent()) || StringUtils.isBlank(sysCaseQuestion.getTitle())) {
            return false;
        }
        if (StringUtils.isBlank(sysCaseQuestion.getSource())) {
            return false;
        }
        String title = sysCaseQuestion.getTitle();
        SysCaseQuestionExample example = new SysCaseQuestionExample();
        example.createCriteria().andTitleEqualTo(title).andUnmountStatusEqualTo(false).andOneIdEqualTo(oneId);
        List<SysCaseQuestion> sysCaseQuestions = caseQuestionMapper.selectByExample(example);
        if (!sysCaseQuestions.isEmpty()) {
            return false;
        }

        String uuid = IdUtil.randomUUID();
        sysCaseQuestion.setCaseId(uuid);
        sysCaseQuestion.setCreateTime(new Date());
        sysCaseQuestion.setUnmountStatus(false);
        caseQuestionMapper.insert(sysCaseQuestion);
        //新增绑定关系
        SysTopicAndCourse sysTopicAndCourse = new SysTopicAndCourse();
        sysTopicAndCourse.setId(IdUtil.randomUUID());
        sysTopicAndCourse.setTopicType("1");
        sysTopicAndCourse.setTopicId(uuid);
        sysTopicAndCourse.setOneId(oneId);
        sysTopicAndCourse.setOneName(sysOneCatalogs1.getOneName());
        if (StringUtils.isNotBlank(sysCaseQuestion.getTwoId())) {
            sysTopicAndCourse.setTwoId(sysCaseQuestion.getTwoId());
            sysTopicAndCourse.setTwoName(sysCaseQuestion.getTwoName());
        }
        if (StringUtils.isNotBlank(sysCaseQuestion.getThreeId())) {
            sysTopicAndCourse.setThreeId(sysCaseQuestion.getThreeId());
            sysTopicAndCourse.setThreeName(sysCaseQuestion.getThreeName());
        }
        topicAndCourseMapper.insert(sysTopicAndCourse);
        return true;
    }

    private SysOneCatalog getSysOneCatalogs(String systemOwner) {
        SysOneCatalogExample example = new SysOneCatalogExample();
        example.createCriteria().andOneNameEqualTo(systemOwner);
        List<SysOneCatalog> sysOneCatalogs = sysOneCatalogMapper.selectByExample(example);
        return sysOneCatalogs.get(0);
    }

    //案例题列表查询 schoolId keyword index size systemOwner
    @AnonymousAccess
    @ApiOperation("案例题页面查询")
    @PostMapping("selectCaseList")
    public PageInfo<SysCaseQuestion> selectCaseList(@RequestParam Integer index,
                                                    @RequestParam Integer size,
                                                    @RequestParam(required = false) String keyword,
                                                    @RequestParam(required = false) String systemOwner,
                                                    @RequestParam(required = false) String schoolId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        List<SysCaseQuestion> list = caseQuestionMapper.selectCaseByConditions(keyword, oneId, schoolId,null,null);
        PageInfo pageInfo1 = PageUtil.pageHelper(list, index, size);
        return pageInfo1;
    }

    //案例题详情查询 caseId
    @AnonymousAccess
    @ApiOperation("案例题详细内容展示")
    @PostMapping("selectCaseDetails")
    public SysCaseQuestion selectCaseByConditionsByID(@RequestParam String caseId,
                                                      @RequestParam String systemOwner) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andOneIdEqualTo(sysOneCatalogs.getOneId()).andTopicIdEqualTo(caseId);
        List<SysTopicAndCourse> sysTopicAndCourses = topicAndCourseMapper.selectByExample(example);
        if (sysTopicAndCourses != null && !sysTopicAndCourses.isEmpty()) {
            SysTopicAndCourse sysTopicAndCourse = sysTopicAndCourses.get(0);
            SysCaseQuestion sysCaseQuestion = caseQuestionMapper.selectByPrimaryKey(caseId);
            sysCaseQuestion.setTwoId(sysTopicAndCourse.getTwoId());
            sysCaseQuestion.setTwoName(sysTopicAndCourse.getTwoName());
            sysCaseQuestion.setThreeId(sysTopicAndCourse.getThreeId());
            sysCaseQuestion.setThreeName(sysTopicAndCourse.getThreeName());
            return sysCaseQuestion;
        }
        return null;
    }


    //案例题删除 caseId
    @AnonymousAccess
    @ApiOperation("案例题删除") //逻辑删除
    @PostMapping("caseDelete")
    public Boolean deleteCase(@RequestParam String caseId, @RequestParam String systemOwner) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        List<SysTopicAndCourse> sysTopicAndCourses = getSysTopicAndCourses(caseId, sysOneCatalogs.getOneId());
        SysCaseQuestion sysCaseQuestion = caseQuestionMapper.selectByPrimaryKey(caseId);
        String source = sysCaseQuestion.getSource();
        if (!"管理员".equals(source)) {
            if (!sysTopicAndCourses.isEmpty()) {
                topicAndCourseMapper.deleteByPrimaryKey(sysTopicAndCourses.get(0).getId());
            }
            SysCaseQuestion dataSysCaseQuestion = caseQuestionMapper.selectByPrimaryKey(sysCaseQuestion.getCaseId());
            dataSysCaseQuestion.setUnmountStatus(true);
            caseQuestionMapper.updateByPrimaryKey(dataSysCaseQuestion);
            return true;
        } else {
            return false;
        }
    }


    private List<SysTopicAndCourse> getSysTopicAndCourses(String caseId, String oneId) {
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andTopicIdEqualTo(caseId).andOneIdEqualTo(oneId).andTopicTypeEqualTo("1");
        List<SysTopicAndCourse> sysTopicAndCourses = topicAndCourseMapper.selectByExample(example);
        return sysTopicAndCourses;
    }


    private List<SysTopicAndCourse> getSysTopicAndCourses(String caseId) {
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andTopicIdEqualTo(caseId).andTopicTypeEqualTo("1");
        List<SysTopicAndCourse> sysTopicAndCourses = topicAndCourseMapper.selectByExample(example);
        return sysTopicAndCourses;
    }


    //案例题编辑 SysCaseQuestion
    @AnonymousAccess
    @ApiOperation("案例题编辑")
    @PostMapping("updateCase")
    public Boolean updateCase(@RequestBody SysCaseQuestion sysCaseQuestion) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(sysCaseQuestion.getOneName());
        SysCaseQuestion sysCaseQuestion1 = caseQuestionMapper.selectByPrimaryKey(sysCaseQuestion.getCaseId());
        if (sysCaseQuestion1 == null) {
            return false;
        }
        if (!sysCaseQuestion.getSource().equals(sysCaseQuestion1.getSource())) {   //老师不能修改管理员上传的，不同直接返回
            return false;
        } else {
            SysTopicAndCourseExample example = new SysTopicAndCourseExample();
            example.createCriteria().andOneIdEqualTo(sysOneCatalogs.getOneId())
                    .andTopicIdEqualTo(sysCaseQuestion.getCaseId())
                    .andTopicTypeEqualTo("1");
            List<SysTopicAndCourse> sysTopicAndCourses = topicAndCourseMapper.selectByExample(example);
            if (!sysTopicAndCourses.isEmpty()) {
                SysTopicAndCourse sysTopicAndCourse = sysTopicAndCourses.get(0);
                sysTopicAndCourse.setTwoId(sysCaseQuestion.getTwoId());
                sysTopicAndCourse.setTwoName(sysCaseQuestion.getTwoName());
                topicAndCourseMapper.updateByPrimaryKeySelective(sysTopicAndCourse);
            }
            caseQuestionMapper.updateByPrimaryKeySelective(sysCaseQuestion);
            return true;
        }
    }


    //案例题步骤信息查询 caseId
    @AnonymousAccess
    @ApiOperation("案例步骤展示")
    @PostMapping("selectCaseStepDetailByCaseId")
    public List<SysCaseQuestionStepWithBLOBs> selectCaseStep(@ApiParam("案例题ID") @RequestParam String caseId) {
        SysCaseQuestionStepExample example = new SysCaseQuestionStepExample();
        example.createCriteria().andCaseIdEqualTo(caseId);
        example.setOrderByClause("sort ASC");
        return caseQuestionStepMapper.selectByExampleWithBLOBs(example);
    }


    //案例题步骤新增 SysCaseQuestionStep
    @AnonymousAccess
    @ApiOperation("案例题添加步骤")
    @PostMapping("insertCaseStep")
    public Boolean insertCaseStepByCaseId(@RequestBody SysCaseQuestionStepWithBLOBs caseQuestionStep) {
        if (caseQuestionStep.getCaseId().isEmpty()) {
            return false;
        }
        if (caseQuestionStep.getTitle().isEmpty()
                || caseQuestionStep.getQuestion().isEmpty()
                || caseQuestionStep.getAnswer().isEmpty()
                || caseQuestionStep.getContent().isEmpty()) {
            return false;
        }
        try {
            caseQuestionStep.setCaseStepId(String.valueOf(UUID.randomUUID()));
            caseQuestionStepMapper.insert(caseQuestionStep);
            return true;
        } catch (Exception e) {
            return false;
        }
    }


    //案例步骤编辑 SysCaseQuestionStepWithBLOBs
    @AnonymousAccess
    @ApiOperation("案例题步骤编辑")
    @PostMapping("updateCaseStep")
    public Boolean updateCaseStep(@RequestBody SysCaseQuestionStepWithBLOBs caseQuestionStepWithBLOBs) {
        List<SysTopicAndCourse> sysTopicAndCourses = getSysTopicAndCourses(caseQuestionStepWithBLOBs.getCaseId());
        if (sysTopicAndCourses.isEmpty()) {
            return false;
        } else {
            caseQuestionStepMapper.updateByPrimaryKeySelective(caseQuestionStepWithBLOBs);
            return true;
        }
    }

    //案例步骤删除 caseStepId
    @AnonymousAccess
    @ApiOperation("案例题步骤删除")
    @PostMapping("deleteCaseStep")
    public Boolean deleteCaseStep(@RequestParam String caseStepId) {
        SysCaseQuestionStepWithBLOBs sysCaseQuestionStepWithBLOBs = caseQuestionStepMapper.selectByPrimaryKey(caseStepId);
        if (sysCaseQuestionStepWithBLOBs == null) {
            return false;
        } else {
            caseQuestionStepMapper.deleteByPrimaryKey(caseStepId);
            return true;
        }
    }

    //案例题步骤内容查看 caseStepId
    @PostMapping("selectCaseStepDetails")
    @ApiOperation("案例题步骤内容查看")
    @AnonymousAccess
    public SysCaseQuestionStepWithBLOBs selectCaseStepDetails(@ApiParam("案例题步骤ID") @RequestParam String caseStepId) {
        return caseQuestionStepMapper.selectByPrimaryKey(caseStepId);
    }


    /**
     * 根据三级ID查询案例题集合
     * 方法名：selectCaseByChapterId
     * 参数：threeId
     * return:List<SysCaseQuestion>
     */
    //案例题步骤内容查看 caseStepId
    @PostMapping("selectCaseByChapterId")
    @ApiOperation("根据三级ID查询案例题集合")
    @AnonymousAccess
    public List<SysCaseQuestion> selectCaseByChapterId(@RequestParam String threeId) {
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andThreeIdEqualTo(threeId);
        List<SysTopicAndCourse> sysTopicAndCourses = topicAndCourseMapper.selectByExample(example);

        List<String> collect = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());

        SysCaseQuestionExample example1 = new SysCaseQuestionExample();
        example1.createCriteria().andCaseIdIn(collect).andSourceEqualTo("管理员");
        List<SysCaseQuestion> list = caseQuestionMapper.selectByExampleWithBLOBs(example1);
        return list;
    }


    /**
     * 根据案例题IDList 查询案例题步骤并排序
     * 方法名：selectCaseStepListBatchByIdListAndSort
     * 参数：caseIdList
     * return:List<TestTestSysCaseQuestionStepWithBLOBs>
     */
    @PostMapping("selectCaseStepListBatchByIdListAndSort")
    @ApiOperation("查询案例题步骤并排序")
    @AnonymousAccess
    public List<SysCaseQuestionStepWithBLOBs> selectCaseStepListBatchByIdListAndSort(@RequestBody List<String> caseIdList) {
        SysCaseQuestionStepExample example = new SysCaseQuestionStepExample();
        example.createCriteria().andCaseIdIn(caseIdList);
        example.setOrderByClause("sort");
        List<SysCaseQuestionStepWithBLOBs> sysCaseQuestionStepWithBLOBs = caseQuestionStepMapper.selectByExampleWithBLOBs(example);
        return sysCaseQuestionStepWithBLOBs;
    }


    /**
     * 根据章节IdList查询案例题
     * 方法名：selectCaseByChapterIdList
     * 参数：List<String> chapterIdList
     * return:List<SysCaseQuestion>
     */
    @PostMapping("selectCaseByChapterIdList")
    @ApiOperation("根据章节IdList查询案例题")
    @AnonymousAccess
    public List<SysCaseQuestion> selectCaseByChapterIdList(@RequestBody List<String> chapterIdList) {
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andThreeIdIn(chapterIdList);
        List<SysTopicAndCourse> sysTopicAndCourses = topicAndCourseMapper.selectByExample(example);
        List<String> idList = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());
        SysCaseQuestionExample sysCaseQuestionExample = new SysCaseQuestionExample();
        sysCaseQuestionExample.createCriteria().andCaseIdIn(idList);
        return caseQuestionMapper.selectByExampleWithBLOBs(sysCaseQuestionExample);
    }


    //单独接口
    //*考试模式--成绩报告案例题question0riginal所属章节，contentOriginal为考核点数量,Sort为序号
    @PostMapping("getGradeReportCase")
    @ApiOperation("考试模式/成绩报告/案例题")
    @AnonymousAccess
    public List<SysCaseQuestionStepWithBLOBs> getGradeReportCase(@RequestBody List<String> caseIdList,
                                                                 @RequestParam String schoolId,
                                                                 @RequestParam String systemOwner) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        return caseQuestionStepMapper.getGradeReportCase(caseIdList, schoolId, oneId);
    }


    //根据课程ID查询案例题
    @PostMapping("selectCaseByCourseId")
    @ApiOperation("根据课程ID查询案例题")
    @AnonymousAccess  //只查学校新增的
    public List<SysCaseQuestion> selectCaseByCourseId(@RequestParam String courseId,
                                                      @RequestParam String systemOwner,
                                                      @RequestParam String schoolId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();

        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andTwoIdEqualTo(courseId).andOneIdEqualTo(oneId);
        List<SysTopicAndCourse> sysTopicAndCourses = topicAndCourseMapper.selectByExample(example);
        if (!sysTopicAndCourses.isEmpty()) {
            List<String> collect = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());
            SysCaseQuestionExample example1 = new SysCaseQuestionExample();
            example1.createCriteria().andCaseIdIn(collect).andSourceEqualTo(schoolId);
            List<SysCaseQuestion> list = caseQuestionMapper.selectByExampleWithBLOBs(example1);
            return list;
        }
        return null;
    }

    //* 单独接口
//* 展示案例题详细信息
    @AnonymousAccess
    @ApiOperation("展示案例题详细信息")
    @PostMapping("selectTrainingByIds")
    public List<TrainingDto> selectTrainingByIds(@RequestBody List<String> caseStepIdList) {
        List<TrainingDto> trainingDtos = caseQuestionStepMapper.selectTrainingByIds(caseStepIdList);
        return trainingDtos;
    }

    //* 单独接口
//* 获取所有内置案例题步骤信息
    @AnonymousAccess
    @ApiOperation("获取所有内置案例题步骤信息")
    @PostMapping("selectAllStepBySystemOwner")
    public List<SysCaseQuestionStepWithBLOBs> selectAllStepBySystemOwner(@RequestParam String systemOwner) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        List<String> ids = topicAndCourseMapper.getCaseIdsBySourceIsAdmin(sysOneCatalogs.getOneId());
        return caseQuestionStepMapper.getCaseStepByIds(ids);
    }


    //* 单独接口
    @AnonymousAccess
    @ApiOperation("获取三级ID和案例题ID")
    @PostMapping("getMapChapterIdAndCaseIdsByCourseIdAndChapterId")
    public Map<String, List<String>> getMapChapterIdAndCaseIdsByCourseIdAndChapterId(@RequestBody List<SysThreeCatalog> sysThreeCatalogs,
                                                                                     @RequestParam String systemOwner) {
        // 批量查询案例题ID
        Map<String, List<String>> caseIdMap = new HashMap<>();
        List<String> courseIdList = new ArrayList<>();
        List<String> chapterIdList = new ArrayList<>();
        for (SysThreeCatalog chapter : sysThreeCatalogs) {
            courseIdList.add(chapter.getTwoId());
            chapterIdList.add(chapter.getThreeId());
        }

        List<Map<String, Object>> results = topicAndCourseMapper.selectCaseIdByCourseAndChapterIds(courseIdList, chapterIdList);
        for (Map<String, Object> result : results) {
            String chapterId = (String) result.get("three_id");
            String caseId = (String) result.get("case_id");
            if (!caseIdMap.containsKey(chapterId)) {
                caseIdMap.put(chapterId, new ArrayList<>());
            }
            caseIdMap.get(chapterId).add(caseId);
        }
        return caseIdMap;
    }
}
