package com.sztzjy.resource_center.controller.api;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageInfo;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.*;
import com.sztzjy.resource_center.entity.dto.SysObjectiveQuestionsDto;
import com.sztzjy.resource_center.mapper.*;
import com.sztzjy.resource_center.util.PageUtil;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@Api(tags = "客观题方面API")
@RequestMapping("api/sys/ObjectiveApi")
public class ObjectiveApi {
    @Autowired
    private SysObjectiveQuestionsMapper sysObjectiveQuestionMapper;
    @Autowired
    private SysTopicAndCourseMapper sysTopicAndCourseMapper;
    @Autowired
    SysOneCatalogMapper oneCatalogMapper;
    @Autowired
    IFileUtil fileUtil;
    @Value("${file.path}")
    private String filePath;
    @Autowired
    private SysTwoCatalogMapper sysTwoCatalogMapper;
    @Autowired
    private SysThreeCatalogMapper sysThreeCatalogMapper;

    private SysOneCatalog getSysOneCatalogs(String systemOwner) {
        SysOneCatalogExample example = new SysOneCatalogExample();
        example.createCriteria().andOneNameEqualTo(systemOwner);
        List<SysOneCatalog> sysOneCatalogs = oneCatalogMapper.selectByExample(example);
        return sysOneCatalogs.get(0);
    }


    /**
     * 客观题新增
     * 方法名：insertObjective
     * 参数：SysObjectiveQuestions systemOwner
     * return: boolean
     */
    @AnonymousAccess
    @ApiOperation("新增单个试题")  //新增 题干不允许重复
    @PostMapping("insertObjective")
    private Boolean insertObjective(@RequestBody SysObjectiveQuestionsDto objectiveQuestion) {
        if (StringUtils.isBlank(objectiveQuestion.getContent())) {
            return false;
        }
        if (StringUtils.isBlank(objectiveQuestion.getAnswer())) {
            return false;
        }
        if (objectiveQuestion.getScore() == null) {
            return false;
        }
        if (StringUtils.isBlank(objectiveQuestion.getType())) {
            return false;
        }
        if (StringUtils.isBlank(objectiveQuestion.getOneName())) {
            return false;
        }
        if (objectiveQuestion.getType().equals("1") || objectiveQuestion.getType().equals("0")) {
            if (StringUtils.isBlank(objectiveQuestion.getQuestionA()) || StringUtils.isBlank(objectiveQuestion.getQuestionB())
                    || StringUtils.isBlank(objectiveQuestion.getQuestionC()) || StringUtils.isBlank(objectiveQuestion.getQuestionD()))
                return false;
        }
        if (objectiveQuestion.getType().equals("2")) {
            if (StringUtils.isBlank(objectiveQuestion.getQuestionA()) || StringUtils.isBlank(objectiveQuestion.getQuestionB()))
                return false;
        }

        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(objectiveQuestion.getOneName());
        String oneId = sysOneCatalogs.getOneId();
        //新增题库表
        String uuid = IdUtil.randomUUID();
        objectiveQuestion.setObjectiveId(uuid);
        objectiveQuestion.setCreateTime(new Date());
        objectiveQuestion.setIsDelete(false);
        objectiveQuestion.setOneID(oneId);
        sysObjectiveQuestionMapper.insert(objectiveQuestion);

        //新增题库关系表
        SysTopicAndCourse sysTopicAndCourse = new SysTopicAndCourse();
        sysTopicAndCourse.setId(IdUtil.randomUUID());
        sysTopicAndCourse.setTopicId(uuid);
        sysTopicAndCourse.setTopicType(objectiveQuestion.getType());
        sysTopicAndCourse.setOneId(oneId);
        sysTopicAndCourse.setOneName(objectiveQuestion.getOneName());
        if (StringUtils.isNotBlank(objectiveQuestion.getTwoID())) {
            sysTopicAndCourse.setTwoId(objectiveQuestion.getTwoID());
            sysTopicAndCourse.setTwoName(objectiveQuestion.getTwoName());
        }
        if (StringUtils.isNotBlank(objectiveQuestion.getThreeID())) {
            sysTopicAndCourse.setThreeId(objectiveQuestion.getThreeID());
            sysTopicAndCourse.setThreeName(objectiveQuestion.getThreeName());
        }
        sysTopicAndCourseMapper.insert(sysTopicAndCourse);
        return true;
    }


    @AnonymousAccess
    @ApiOperation("客观题编辑")
    @PostMapping("updateObjective")
    private Boolean updateObjective(@RequestBody SysObjectiveQuestionsDto dto,
                                    @RequestParam String systemOwner,
                                    @RequestParam String schoolId) {
        //ID不为空，修改绑定信息
        if (StringUtils.isNotBlank(dto.getTopicAndCourseId())) {
            SysTopicAndCourse sysTopicAndCourse = new SysTopicAndCourse();
            sysTopicAndCourse.setId(dto.getTopicAndCourseId());
            sysTopicAndCourse.setTopicId(dto.getObjectiveId());
            sysTopicAndCourse.setOneId(dto.getOneID());
            sysTopicAndCourse.setOneName(dto.getOneName());
            sysTopicAndCourse.setTwoId(dto.getTwoID());
            sysTopicAndCourse.setTwoName(dto.getTwoName());
            sysTopicAndCourseMapper.updateByPrimaryKeySelective(sysTopicAndCourse);
        }
        //修改题库基础信息
        SysObjectiveQuestions newData = new SysObjectiveQuestions();
        BeanUtils.copyProperties(dto, newData);
        sysObjectiveQuestionMapper.updateByPrimaryKeySelective(newData);
        return true;
    }


    /**
     * 客观题批量删除
     * 方法名：batchDeleteObjective
     * 参数：List<String> objIdList  schoolId
     * return: boolean
     */
    @AnonymousAccess
    @ApiOperation("客观题批量删除")
    @PostMapping("batchDeleteObjective")
    private Boolean batchDeleteObjective(@RequestBody List<String> objIdList,
                                         @RequestParam String schoolId) {
        SysObjectiveQuestionsExample example = new SysObjectiveQuestionsExample();
        example.createCriteria().andObjectiveIdIn(objIdList);
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example);

        List<String> ids = new ArrayList<>();
        for (SysObjectiveQuestions sysObjectiveQuestion : sysObjectiveQuestions) {
            if (sysObjectiveQuestion.getSource().equals(schoolId)) {
                ids.add(sysObjectiveQuestion.getObjectiveId());
            } else {
                continue;
            }
        }
        try {
            sysObjectiveQuestionMapper.batchDelete(ids);
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * 客观题列表查看
     * 方法名：selectObjectiveList
     * 参数：schoolId courseId type content index size
     * return:PageInfo<SysObjectiveQuestions>
     */
    @AnonymousAccess
    @ApiOperation("客观题列表查看")
    @PostMapping("selectObjectiveList")  //todo 关于老师删除的题库如何查询
    private PageInfo<SysObjectiveQuestionsDto> selectObjectiveList(@RequestParam Integer index,
                                                                   @RequestParam Integer size,
                                                                   @RequestParam(required = false) String courseId,
                                                                   @RequestParam(required = false) String chapterId,
                                                                   @RequestParam(required = false) String type,
                                                                   @RequestParam String schoolId,
                                                                   @ApiParam("题干") @RequestParam(required = false) String content,
                                                                   @RequestParam String systemOwner) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        List<SysObjectiveQuestionsDto> list = sysObjectiveQuestionMapper.getTopicByConfig(oneId, courseId, chapterId, type, schoolId, content);
        PageInfo pageInfo = PageUtil.pageHelper(list, index, size);
        return (pageInfo);
    }

    /**
     * 客观题单个详情查看
     * 方法名：selectObjectiveDetails
     * 参数：objectiveId
     * return:SysObjectiveQuestions
     */
    @AnonymousAccess
    @ApiOperation("客观题单个详情查看")
    @PostMapping("selectObjectiveDetails")
    private SysObjectiveQuestionsDto selectObjectiveDetails(@RequestParam String objectiveId,
                                                            @RequestParam String systemOwner,
                                                            @RequestParam String schoolId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);

        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andTopicIdEqualTo(objectiveId).andOneIdEqualTo(sysOneCatalogs.getOneId());
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);
        SysObjectiveQuestionsDto dto = new SysObjectiveQuestionsDto();

        if (sysTopicAndCourses != null && !sysTopicAndCourses.isEmpty()) {
            SysTopicAndCourse sysTopicAndCourse = sysTopicAndCourses.get(0);
            dto.setOneID(sysTopicAndCourse.getOneId());
            dto.setOneName(sysTopicAndCourse.getOneName());
            dto.setTwoID(sysTopicAndCourse.getTwoId());
            dto.setTwoName(sysTopicAndCourse.getTwoName());
            dto.setThreeName(sysTopicAndCourse.getThreeName());
            dto.setThreeID(sysTopicAndCourse.getThreeId());
        }
        SysObjectiveQuestions sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByPrimaryKey(objectiveId);
        BeanUtils.copyProperties(sysObjectiveQuestions, dto);
        return dto;
    }

    /**
     * 客观题导入模板下载
     * 方法名：downloadResource
     * 参数：response
     * return:void
     */
    @PostMapping("downloadResource")
    @ApiOperation("客观题导入模板下载")
    @AnonymousAccess
    public void downloadResource(HttpServletResponse response) {
        fileUtil.download(response, "客观题导入模板.xlsx", "/客观题导入模板.xlsx");
    }

    /**
     * 客观题批量导入
     * 方法名：insertObjectiveByExcel
     * 参数：MultipartFile schoolId
     * return:Boolean
     */
    @AnonymousAccess
    @ApiOperation("本地excel导入试题")
    @PostMapping("insertObjectiveByExcel")
    @Transactional
     Boolean importTopicByLocal(@RequestParam MultipartFile file,
                                @RequestParam String schoolId) throws IOException, InvalidFormatException {
        if (file == null) {
            return false;
        }
        return insertObjectiveByExcel(file, schoolId);
    }

    //如果是内置增加 则需要提供课程和章节信息
    //如果是老师增加 则只需要提供课程信息
    public Boolean insertObjectiveByExcel(MultipartFile file, String schoolId) throws IOException, InvalidFormatException {
        List<SysObjectiveQuestions> objectiveQuestionList = new ArrayList<>();
        List<SysTopicAndCourse> topicAndCourseList = new ArrayList<>();


        Workbook workbook = WorkbookFactory.create(file.getInputStream());
        Sheet sheet = workbook.getSheetAt(0);

        //创建三级的ID和名称Map
        Map<String, String> oneIdMap = new HashMap<>();
        Map<String, String> twoIdMap = new HashMap<>();
        Map<String, String> threeIdMap = new HashMap<>();

        // 迭代每一行
        Iterator<Row> iterator = sheet.iterator();
        iterator.next();
        while (iterator.hasNext()) {
            Row row = iterator.next();

            /*解析列，下标从0开始*/
            Cell cell0 = row.getCell(0);
            Cell cell1 = row.getCell(1);
            Cell cell2 = row.getCell(2);
            Cell cell3 = row.getCell(3);
            Cell cell4 = row.getCell(4);
            Cell cell5 = row.getCell(5);
            Cell cell6 = row.getCell(6);
            Cell cell7 = row.getCell(7);
            Cell cell8 = row.getCell(8);
            Cell cell9 = row.getCell(9);
            Cell cell10 = row.getCell(10);
            Cell cell11 = row.getCell(11);
            Cell cell12 = row.getCell(12);
            //判断题的BCDE选项和解析可能为空
            if (cell0 == null || cell1 == null || cell2 == null || cell6 == null || cell7 == null || cell9 == null) {
                return false;
            }

            String content = this.getCellValue(cell0);//题干
            String a = this.getCellValue(cell1); //选项A
            String b = this.getCellValue(cell2);//选项B
            String type = this.getCellValue(cell6);//题型
            String answer = this.getCellValue(cell7);//答案
            String score = this.getCellValue(cell9);//分值
            String oneName = this.getCellValue(cell10);//一级目录名称
            String twoName = this.getCellValue(cell11);//二级目录名称
            String threeName = this.getCellValue(cell12);//三级目录名称
            String c = cell3 != null ? this.getCellValue(cell3) : null;
            String d = cell4 != null ? this.getCellValue(cell4) : null;
            String e = cell5 != null ? this.getCellValue(cell5) : null;
            String analyze = cell8 != null ? this.getCellValue(cell8) : null;


            SysObjectiveQuestions obj = new SysObjectiveQuestions();
            String topicId = IdUtil.randomUUID();
            obj.setObjectiveId(topicId);
            obj.setContent(content);
            obj.setType(type);
            obj.setSource(schoolId);
            obj.setQuestionA(a);
            obj.setQuestionB(b);
            obj.setQuestionC(c);
            obj.setQuestionD(d);
            obj.setQuestionE(e);
            obj.setAnswer(answer);
            obj.setAnalysis(analyze);
            obj.setScore(BigDecimal.valueOf(Integer.parseInt(score)));
            obj.setCreateTime(new Date());
            obj.setIsDelete(false);
            objectiveQuestionList.add(obj);

            //判断名称是否被查出来过
            String oneId = oneIdMap.get(oneName);
            if (oneId == null) {
                oneId = oneCatalogMapper.getIdByName(oneName);
                oneIdMap.put(oneName, oneId);
            }

            String twoId = twoIdMap.get(twoName);
            if (twoId == null) {
                twoId = sysTwoCatalogMapper.getIdByName(twoName);
                twoIdMap.put(twoName, twoId);
            }

            String threeId = threeIdMap.get(threeName);
            if (threeId == null) {
                threeId = sysThreeCatalogMapper.getIdByName(threeName,twoName);
                threeIdMap.put(threeName, threeId);
            }


            SysTopicAndCourse sysTopicAndCourse = new SysTopicAndCourse();
            sysTopicAndCourse.setId(IdUtil.randomUUID());
            sysTopicAndCourse.setTopicId(topicId);
            sysTopicAndCourse.setTopicType("0");
            sysTopicAndCourse.setOneName(oneName);
            sysTopicAndCourse.setOneId(oneId);
            sysTopicAndCourse.setTwoName(twoName);
            sysTopicAndCourse.setTwoId(twoId);
            sysTopicAndCourse.setThreeId(threeId);
            sysTopicAndCourse.setThreeName(threeName);
            topicAndCourseList.add(sysTopicAndCourse);
        }
        sysTopicAndCourseMapper.batchInsert(topicAndCourseList);
        int result = sysObjectiveQuestionMapper.insertBatch(objectiveQuestionList);
        if (result > 0) {
            return true;
        } else {
            return false;
        }
    }


    public String getCellValue(Cell cell) {
        if (cell.getCellType() == Cell.CELL_TYPE_BOOLEAN) {
            return String.valueOf(cell.getBooleanCellValue());
        } else if (cell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
            Double d = cell.getNumericCellValue();
            return String.valueOf(d.intValue());
        }
        return String.valueOf(cell.getStringCellValue());
    }


    /**
     * 根据客观题IDList批量查询客观题
     * 方法名：selectBatchByIdList
     * 参数：objectIdList
     * return:List<SysObjectiveQuestions>
     */
    @AnonymousAccess
    @ApiOperation("根据客观题IDList批量查询客观题")
    @PostMapping("selectBatchByIdList")
    private List<SysObjectiveQuestions> selectBatchByIdList(@RequestBody List<String> objectIdList) {
        SysObjectiveQuestionsExample example = new SysObjectiveQuestionsExample();
        example.createCriteria().andObjectiveIdIn(objectIdList);
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example);
        return sysObjectiveQuestions;
    }

    /**
     * 根据章节ID 随机获取十道  内置  题目
     * 方法名：selectRandomObjectiveByChapterId
     * 参数：chapterId
     * return:List<SysObjectiveQuestions>
     */
    @AnonymousAccess
    @ApiOperation("根据章节ID随机获取十道内置题目")
    @PostMapping("selectRandomObjectiveByChapterId")
    private List<SysObjectiveQuestions> selectRandomObjectiveByChapterId(@RequestParam String chapterId) {
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andThreeIdEqualTo(chapterId);
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);
        List<String> ids = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());

        SysObjectiveQuestionsExample example1 = new SysObjectiveQuestionsExample();
        example1.createCriteria().andObjectiveIdIn(ids).andSourceEqualTo("管理员");
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example1);
        Collections.shuffle(sysObjectiveQuestions);
        return sysObjectiveQuestions.stream().limit(10).collect(Collectors.toList());
    }


    /**
     * 根据章节ID 查询所有  内置  题目
     * 方法名：selectObjectQuestionListByChapterId
     * 参数：chapterId
     * return:List<SysObjectiveQuestions>
     */
    @AnonymousAccess
    @ApiOperation("根据章节I查询所有内置题目")
    @PostMapping("selectObjectQuestionListByChapterId")
    private List<SysObjectiveQuestions> selectObjectQuestionListByChapterId(@RequestParam String chapterId) {
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andThreeIdEqualTo(chapterId);
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);
        List<String> ids = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());

        SysObjectiveQuestionsExample example1 = new SysObjectiveQuestionsExample();
        example1.createCriteria().andObjectiveIdIn(ids).andSourceEqualTo("管理员");
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example1);
        return sysObjectiveQuestions;
    }


    /**
     * 随机获取35道  内置  单选题
     * 方法名：selectRandomObjectiveSingle
     * 参数：systemOwner
     * return:List<SysObjectiveQuestions>
     */
    @AnonymousAccess
    @ApiOperation("随机获取35道内置单选题")
    @PostMapping("selectRandomObjectiveSingle")
    private List<SysObjectiveQuestions> selectRandomObjectiveSingle(@RequestParam String systemOwner) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andOneIdEqualTo(oneId);
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);

        List<String> ids = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());

        SysObjectiveQuestionsExample example1 = new SysObjectiveQuestionsExample();
        example1.createCriteria().andObjectiveIdIn(ids).andSourceEqualTo("管理员").andTypeEqualTo("0");
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example1);
        Collections.shuffle(sysObjectiveQuestions);
        return sysObjectiveQuestions.stream().limit(35).collect(Collectors.toList());
    }


    /**
     * 随机获取5道  内置  多选题
     * 方法名：selectRandomObjectiveMany
     * 参数：systemOwner
     * return:List<SysObjectiveQuestions>
     */
    @AnonymousAccess
    @ApiOperation("随机获取5道内置多选题")
    @PostMapping("selectRandomObjectiveMany")
    private List<SysObjectiveQuestions> selectRandomObjectiveMany(@RequestParam String systemOwner) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andOneIdEqualTo(oneId);
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);

        List<String> ids = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());

        SysObjectiveQuestionsExample example1 = new SysObjectiveQuestionsExample();
        example1.createCriteria().andObjectiveIdIn(ids).andSourceEqualTo("管理员").andTypeEqualTo("1");
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example1);
        Collections.shuffle(sysObjectiveQuestions);
        return sysObjectiveQuestions.stream().limit(5).collect(Collectors.toList());
    }


    /**
     * 随机获取10道  内置  判断题
     * 方法名：selectRandomObjectiveJudge
     * 参数：systemOwner
     * return:List<SysObjectiveQuestions>
     */
    @AnonymousAccess
    @ApiOperation("随机获取5道内置多选题")
    @PostMapping("selectRandomObjectiveJudge")
    private List<SysObjectiveQuestions> selectRandomObjectiveJudge(@RequestParam String systemOwner) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andOneIdEqualTo(oneId);
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);

        List<String> ids = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());

        SysObjectiveQuestionsExample example1 = new SysObjectiveQuestionsExample();
        example1.createCriteria().andObjectiveIdIn(ids).andSourceEqualTo("管理员").andTypeEqualTo("2");
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example1);
        Collections.shuffle(sysObjectiveQuestions);
        return sysObjectiveQuestions.stream().limit(10).collect(Collectors.toList());
    }


    /**
     * 获取客观题库单选题数量
     * 方法名：selectCountSingle
     * 参数：schoolId  systemOwner
     * return:Integer
     */
    @AnonymousAccess
    @ApiOperation("获取客观题库单选题数量")
    @PostMapping("selectCountSingle")
    private Integer selectCountSingle(@RequestParam String systemOwner,
                                      @RequestParam String schoolId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andOneIdEqualTo(oneId);
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);

        List<String> ids = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());


        SysObjectiveQuestionsExample example1 = new SysObjectiveQuestionsExample();
        List<String> list = new ArrayList();
        list.add(schoolId);
        list.add("管理员");
        example1.createCriteria().andObjectiveIdIn(ids).andSourceIn(list).andTypeEqualTo("0");
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example1);
        return sysObjectiveQuestions.size();
    }


    /**
     * 获取客观题库多选题数量
     * 方法名：selectCountMany
     * 参数：schoolId  systemOwner
     * return:Integer
     */
    @AnonymousAccess
    @ApiOperation("获取客观题库多选题数量")
    @PostMapping("selectCountMany")
    private Integer selectCountMany(@RequestParam String systemOwner,
                                    @RequestParam String schoolId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andOneIdEqualTo(oneId);
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);

        List<String> ids = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());


        SysObjectiveQuestionsExample example1 = new SysObjectiveQuestionsExample();
        List<String> list = new ArrayList();
        list.add(schoolId);
        list.add("管理员");
        example1.createCriteria().andObjectiveIdIn(ids).andSourceIn(list).andTypeEqualTo("1");
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example1);
        return sysObjectiveQuestions.size();
    }


    /**
     * 获取客观题库判断数量
     * 方法名：selectCountJudge
     * 参数：schoolId  systemOwner
     * return:Integer
     */
    @AnonymousAccess
    @ApiOperation("获取客观题库单选题数量")
    @PostMapping("selectCountJudge")
    private Integer selectCountJudge(@RequestParam String systemOwner,
                                     @RequestParam String schoolId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andOneIdEqualTo(oneId);
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);

        List<String> ids = sysTopicAndCourses.stream().map(SysTopicAndCourse::getTopicId).collect(Collectors.toList());


        SysObjectiveQuestionsExample example1 = new SysObjectiveQuestionsExample();
        List<String> list = new ArrayList();
        list.add(schoolId);
        list.add("管理员");
        example1.createCriteria().andObjectiveIdIn(ids).andSourceIn(list).andTypeEqualTo("2");
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example1);
        return sysObjectiveQuestions.size();
    }


    //*获取所选客观题数量及分数
    @AnonymousAccess
    @ApiOperation("获取所选客观题数量及分数")
    @PostMapping("getChooseObjCountAndSorce")
    private Map<String, BigDecimal> getChooseObjCountAndSorce(@RequestBody List<String> objectIdList) {

        Map<String, BigDecimal> chooseObjCountAndSorce = sysObjectiveQuestionMapper.getChooseObjCountAndSorce(objectIdList);
        return chooseObjCountAndSorce;
    }


    //     *根据客观题类型査询客观题
    @AnonymousAccess
    @ApiOperation("根据客观题类型査询客观题")
    @PostMapping("selectObjByType")
    private List<SysObjectiveQuestionsDto> selectObjByType(@RequestParam String type,
                                                           @RequestParam String schoolId,
                                                           @RequestParam String systemOwner) {
        List<SysObjectiveQuestionsDto> sysObjectiveQuestionsDtos = sysObjectiveQuestionMapper.selectObjByType(type, schoolId);

        return sysObjectiveQuestionsDtos;
    }
}
