package com.sztzjy.resource_center.controller.api;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.*;
import com.sztzjy.resource_center.entity.dto.SysResourceDto;
import com.sztzjy.resource_center.mapper.SysOneCatalogMapper;
import com.sztzjy.resource_center.mapper.SysResourceAndCourseMapper;
import com.sztzjy.resource_center.mapper.SysResourceMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@Api(tags = "资源方面API")
@RequestMapping("api/tea/ResourceCenterApi")
public class ResourceApi {
    @Autowired
    private SysResourceMapper sysResourceMapper;
    @Value("${file.path}")
    private String filePath;
    @Autowired
    private IFileUtil fileUtil;
    @Autowired
    private SysResourceAndCourseMapper sysResourceAndCourseMapper;
    @Autowired
    SysOneCatalogMapper oneCatalogMapper;


    private SysOneCatalog getSysOneCatalogs(String systemOwner) {
        SysOneCatalogExample example = new SysOneCatalogExample();
        example.createCriteria().andOneNameEqualTo(systemOwner);
        List<SysOneCatalog> sysOneCatalogs = oneCatalogMapper.selectByExample(example);
        return sysOneCatalogs.get(0);
    }


    /**
     * 上传资源文件
     * 方法名：uploadResource
     * 参数：MultipartFile schoolId twoId resourceType systemOwner
     * return: boolean
     */
    @AnonymousAccess
    @ApiOperation("上传资源文件")
    @PostMapping("uploadResource")
    public Boolean uploadResource(@RequestParam MultipartFile file,
                                  @RequestParam String schoolId,
                                  @RequestParam String twoId,
                                  @RequestParam String twoName,
                                  @RequestParam String resourceType,
                                  @RequestParam String systemOwner,
                                  @RequestParam String fileName) {
        if (file == null) {
            return false;
        }
        //取到文件后缀名
        int lastIndex = fileName.lastIndexOf(".");
        String extension = fileName.substring(lastIndex);
        SysResource sysResource = new SysResource();
        //判断文件type   //todo 格式可能不够全
        if (extension.equals(".pdf") || extension.equals(".ppt") || extension.equals(".pptx")
                || extension.equals(".xlsx") || extension.equals(".xls")
                || extension.equals(".doc") || extension.equals(".docx")
                || extension.equals(".txt")) {
            sysResource.setResourceType("课件");
        } else if (extension.equals(".mp4") || extension.equals(".wmv") || extension.equals(".avi")) {
            sysResource.setResourceType("视频");
        } else {
            return false;
        }
        String url = fileUtil.upload(file);
        String uuid = IdUtil.randomUUID();
        sysResource.setResourceId(uuid);
        sysResource.setUrl(url);
        sysResource.setSource(schoolId);  //管理员或者 xx学校
        sysResource.setResourceName(fileName);
        sysResource.setStatus(1);
        sysResource.setCreateTime(new Date());  //todo 测试修改资源之后 新增时间会不会变化，有变化就错了
        sysResourceMapper.insert(sysResource);

        //绑定资源的对应关系
        try {
            SysResourceAndCourse sysResourceAndCourse = new SysResourceAndCourse();
            SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
            String oneId = sysOneCatalogs.getOneId();
            sysResourceAndCourse.setId(IdUtil.randomUUID());
            sysResourceAndCourse.setOneId(oneId);
            sysResourceAndCourse.setTwoId(twoId);
            sysResourceAndCourse.setOneName(systemOwner);
            sysResourceAndCourse.setTwoName(twoName);
            sysResourceAndCourse.setResourceId(uuid);  //资源ID
            sysResourceAndCourseMapper.insert(sysResourceAndCourse);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }


    /**
     * 查看资源文件列表
     * 方法名：selectResource
     * 参数：schoolId twoId oneId
     * return: List<SysResource>
     */
    @AnonymousAccess
    @ApiOperation("查看资源文件列表")
    @PostMapping("selectResource")
    public List<SysResourceDto> selectResource(@RequestParam String oneId,  //这里传的是一级的名称
                                               @RequestParam String twoId,
                                               @RequestParam String schoolId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(oneId);
        return sysResourceMapper.selectResource(sysOneCatalogs.getOneId(), twoId, null, schoolId);
    }


    /**
     * 根据章节查看资源文件列表
     * 方法名：selectResourceByChapterId
     * 参数：threeId oneId
     * return: List<SysResource>
     */
    @AnonymousAccess
    @ApiOperation("根据章节查看资源文件列表")
    @PostMapping("selectResourceByChapterId")
    public List<SysResourceDto> selectResourceByChapterId(@RequestParam String oneId,  //这里传的是一级的名称
                                                          @RequestParam String threeId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(oneId);
        return sysResourceMapper.selectResource(sysOneCatalogs.getOneId(), null, threeId, null);
    }


    /**
     * 获取资源类型数量
     * 方法名：getResourceTypeCount
     * 参数：schoolId systemOwner
     * return: Map<String,Integer> 课件：121
     */
    @AnonymousAccess
    @ApiOperation("获取资源类型数量")
    @PostMapping("getResourceTypeCount")
    public Map<String, Long> getResourceTypeCount(@RequestParam String systemOwner,
                                                  @RequestParam String schoolId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        SysResourceAndCourseExample example = new SysResourceAndCourseExample();
        example.createCriteria().andOneIdEqualTo(sysOneCatalogs.getOneId());
        List<SysResourceAndCourse> sysResourceAndCourses = sysResourceAndCourseMapper.selectByExample(example);
        List<String> ids = sysResourceAndCourses.stream().map(SysResourceAndCourse::getResourceId).collect(Collectors.toList());
        List<Map<String, Object>> countByType = sysResourceMapper.getCountByType(ids, schoolId);
        Map<String, Long> resultMap = new HashMap<>();
        for (Map<String, Object> item : countByType) {
            resultMap.put(item.get("resource_type").toString(), (Long) item.get("count"));
        }
        return resultMap;
    }


    /**
     * 获取该系统下 学校上传和内置的全部资源数量
     * 方法名：getTotalResource
     * 参数：systemOwner schoolId
     * return: Integer
     */
    @AnonymousAccess
    @ApiOperation("获取该系统下 学校上传和内置的全部资源数量")
    @PostMapping("getTotalResource")
    public Integer getTotalResource(@RequestParam String systemOwner,
                                    @RequestParam String schoolId) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        SysResourceAndCourseExample example = new SysResourceAndCourseExample();
        example.createCriteria().andOneIdEqualTo(sysOneCatalogs.getOneId());
        List<SysResourceAndCourse> sysResourceAndCourses = sysResourceAndCourseMapper.selectByExample(example);
        List<String> ids = sysResourceAndCourses.stream().map(SysResourceAndCourse::getResourceId).collect(Collectors.toList());

        List<String> list = Arrays.asList(schoolId, "管理员");
        SysResourceExample example1 = new SysResourceExample();
        example1.createCriteria().andResourceIdIn(ids).andResourceIdIn(list);
        List<SysResource> sysResources = sysResourceMapper.selectByExample(example1);
        return sysResources.size();
    }


    //下载资源文件
    @GetMapping("downloadResource")
    @ApiOperation("下载资源文件")
    @AnonymousAccess
    public void downloadResource(@ApiParam("资源ID") @RequestParam String resourceId, HttpServletResponse response) {
        SysResource sysResource = sysResourceMapper.selectByPrimaryKey(resourceId);
        if ("课件".equals(sysResource.getResourceType())) {
            fileUtil.download(response, sysResource.getResourceName(), sysResource.getUrl());
        }
        if (sysResource.getCount() == null) {
            sysResource.setCount(1);
        } else {
            sysResource.setCount(sysResource.getCount() + 1);
        }
        sysResourceMapper.updateByPrimaryKey(sysResource);
    }


    @GetMapping("/getResource")
    @ApiOperation("预览/获取视频或课件")
    @AnonymousAccess
    public ResponseEntity<FileSystemResource> getResource(@ApiParam("资源ID") @RequestParam String id) {

        SysResource sysResource = sysResourceMapper.selectByPrimaryKey(id);
        String url = sysResource.getUrl();
        String videoPath = filePath + url;
        File videoFile = new File(videoPath);
        if ("视频".equals(sysResource.getResourceType())) {
            if (videoFile.exists()) {
//                    Path path = Paths.get(videoPath);
                FileSystemResource fileSystemResource = new FileSystemResource(videoFile);

                //设置count
                if (sysResource.getCount() == null) {
                    sysResource.setCount(1);
                } else {
                    sysResource.setCount(sysResource.getCount() + 1);
                }
                sysResourceMapper.updateByPrimaryKey(sysResource);


                return ResponseEntity.ok()
                        .contentType(MediaType.parseMediaType("video/mp4"))
                        .body(fileSystemResource);
            } else {
                return ResponseEntity.notFound().build();
            }
        } else if ("课件".equals(sysResource.getResourceType())) {
            if (videoFile.exists()) {
                FileSystemResource fileSystemResource = new FileSystemResource(videoFile);

                //设置count
                if (sysResource.getCount() == null) {
                    sysResource.setCount(1);
                } else {
                    sysResource.setCount(sysResource.getCount() + 1);
                }
                sysResourceMapper.updateByPrimaryKey(sysResource);


                if (url.endsWith("pptx")) {
                    return ResponseEntity.ok()
                            .contentType(MediaType.parseMediaType("application/vnd.openxmlformats-officedocument.presentationml.presentation"))
                            .body(fileSystemResource);
                } else if (url.endsWith("ppt")) {
                    return ResponseEntity.ok()
                            .contentType(MediaType.parseMediaType("application/vnd.ms-powerpoint"))
                            .body(fileSystemResource);
                } else {
                    return ResponseEntity.ok()
                            .contentType(MediaType.parseMediaType("application/pdf"))
                            .body(fileSystemResource);
                }
            } else {
                return ResponseEntity.notFound().build();
            }
        }

        return ResponseEntity.notFound().build();
    }

    @AnonymousAccess
    @ApiOperation("学习资源/删除") //同步过来的不做删除
    @PostMapping("deleteResource")
    public ResultEntity<String> deleteResource(@RequestParam String id,
                                               @Param("学校ID") @RequestParam String source) {
        SysResource sysResource = sysResourceMapper.selectByPrimaryKey(id);
        if (!sysResource.getSource().equals(source)) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "删除失败!");
        }
        SysResourceAndCourseExample example = new SysResourceAndCourseExample();
        example.createCriteria().andResourceIdEqualTo(id);
        sysResourceAndCourseMapper.deleteByExample(example);
        sysResourceMapper.deleteByPrimaryKey(id);
        return new ResultEntity<>(HttpStatus.OK, "删除成功!");
    }


    /**
     * 获取最受欢迎的资源名称
     * 方法名：getMostPopularName
     * 参数：schoolId systemOwner
     * return: String
     */
    @AnonymousAccess
    @ApiOperation("获取最受欢迎的资源名称")
    @PostMapping("getMostPopularName")
    public String getMostPopularName(@RequestParam String systemOwner,
                                     @Param("学校ID") @RequestParam String schoolId) {
        return sysResourceMapper.getMostPopularName();
    }
}
