package com.sztzjy.resource_center.controller.api;

import cn.hutool.core.util.CharsetUtil;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.SysCaseQuestion;
import com.sztzjy.resource_center.entity.SysResourceData;
import com.sztzjy.resource_center.entity.SysResourceDataExample;
import com.sztzjy.resource_center.mapper.SysCaseQuestionMapper;
import com.sztzjy.resource_center.mapper.SysResourceDataMapper;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

@RestController
@Api(tags = "数据集方面API")
@RequestMapping("api/tea/ResourceApi")
public class ResourceDataApi {

    @Autowired
    private SysResourceDataMapper sysResourceDataMapper;
    @Autowired
    private SysCaseQuestionMapper caseQuestionMapper;
    @Autowired
    IFileUtil fileUtil;


    /**
     * 数据集文件上传
     * 方法名：uploadResourceData
     * 参数：MultipartFile schoolId caseId
     * return: boolean
     */
    @AnonymousAccess
    @ApiOperation("数据集文件上传")
    @PostMapping("uploadResourceData")
    private Boolean uploadResourceData(@RequestParam MultipartFile file,
                                       @RequestParam String schoolId,
                                       @RequestParam String fileName,
                                       @RequestParam String caseId) {
        SysCaseQuestion sysCaseQuestion = caseQuestionMapper.selectByPrimaryKey(caseId);
        if (!schoolId.equals(sysCaseQuestion.getSource())) {
            return false;
        }
        try {
            String uploadUrl = fileUtil.upload(file);
            SysResourceData resourceData = new SysResourceData();
            resourceData.setResourceDataId(String.valueOf(UUID.randomUUID()));
            resourceData.setResourceName(fileName);
            resourceData.setSize((double) file.getSize());
            resourceData.setSource(schoolId);
            resourceData.setCaseId(caseId);
            resourceData.setUrl(uploadUrl);
            sysResourceDataMapper.insert(resourceData);
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    /**
     * 数据集列表信息查询
     * 方法名：selectResourceDataList
     * 参数：caseId
     * return: List<SysResourceData>
     */
    @AnonymousAccess
    @ApiOperation("数据集列表信息查询")
    @PostMapping("selectResourceDataList")
    private List<SysResourceData> selectResourceDataList(@RequestParam String caseId) {
        SysResourceDataExample example = new SysResourceDataExample();
        example.createCriteria().andCaseIdEqualTo(caseId);
        return sysResourceDataMapper.selectByExample(example);
    }


    /**
     * 数据集删除
     * 方法名：deleteResourceData
     * 参数：resourceId schoolId
     * return: boolean
     */
    @AnonymousAccess
    @ApiOperation("数据集删除")
    @PostMapping("deleteResourceData")
    private Boolean deleteResourceData(@RequestParam String resourceId,
                                       @RequestParam String schoolId) {
        SysResourceData sysResourceData = sysResourceDataMapper.selectByPrimaryKey(resourceId);
        if (!sysResourceData.getSource().equals(schoolId)) {
            return false;
        } else {
            sysResourceDataMapper.deleteByPrimaryKey(resourceId);
            return true;
        }
    }


    /**
     * 数据集下载
     * 方法名：downloadResourceData
     * 参数：resourceId response
     * return: void
     */
    @AnonymousAccess
    @ApiOperation("数据集下载")
    @GetMapping("downloadResourceData")
    private void selectResourceDataList(@RequestParam String resourceId,
                                           HttpServletResponse response) {
        try {
            SysResourceData sysResourceData = sysResourceDataMapper.selectByPrimaryKey(resourceId);
            String resourceName = sysResourceData.getResourceName();
            fileUtil.download(response, resourceName, sysResourceData.getUrl());
        } catch (Exception e) {
        }
    }
}
