package com.sztzjy.resource_center.controller.new_module.admin;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.nimbusds.jose.shaded.gson.Gson;
import com.nimbusds.jose.shaded.gson.JsonSyntaxException;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.admin.*;
import com.sztzjy.resource_center.mapper.admin.AdminDataLabelMapper;
import com.sztzjy.resource_center.mapper.admin.AdminDataMapper;
import com.sztzjy.resource_center.mapper.admin.AdminFileMapper;
import com.sztzjy.resource_center.util.PageUtil;
import com.sztzjy.resource_center.util.ResultEntity;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Author xcj
 * @Date 2024/7/11
 */
@RestController
@Api(tags = "管理员数据集锦页面")
@RequestMapping("api/admin/data")
public class AdminDataController {
    @Autowired
    private IFileUtil fileUtil;
    @Autowired
    private AdminDataLabelMapper adminDataLabelMapper;
    @Autowired
    private AdminFileMapper adminFileMapper;
    @Autowired
    private AdminDataMapper adminDataMapper;

    @PostMapping("/add")
    @ApiOperation("新增")
    @AnonymousAccess
    @Transactional
    ResultEntity<String> add(@RequestParam String adminDataDtoJSON,
                             @RequestParam MultipartFile pictureFile,
                             @RequestParam(required = false) List<MultipartFile> dataFile) {
        Gson gson = new Gson();
        AdminDataDto adminDataDto = null;
        try {
            adminDataDto = gson.fromJson(adminDataDtoJSON, AdminDataDto.class);
        } catch (JsonSyntaxException e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "服务器错误，请联系管理员");
        }
        if (StringUtils.isBlank(adminDataDto.getName())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入名称！");
        }
        if (StringUtils.isBlank(adminDataDto.getFieldDescription())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入字段描述！");
        }
        if (StringUtils.isBlank(adminDataDto.getDataScenarios())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入数据应用场景！");
        }
        if (adminDataDto.getAdminLabelList() == null || adminDataDto.getAdminLabelList().isEmpty()) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "至少选择一个标签！");
        }

        List<AdminLabel> adminLabelList = adminDataDto.getAdminLabelList();

        for (AdminLabel adminLabel : adminLabelList) {
            if (StringUtils.isBlank(adminLabel.getLabelId()) || StringUtils.isBlank(adminLabel.getName()) || StringUtils.isBlank(adminLabel.getType())) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "标签数据不完整！");
            }
        }

        //新增
        AdminDataWithBLOBs adminDataWithBLOBs = new AdminDataWithBLOBs();
        String dataId = IdUtil.randomUUID();
        adminDataWithBLOBs.setDataId(dataId);
        adminDataWithBLOBs.setName(adminDataDto.getName());
        adminDataWithBLOBs.setDataScenarios(adminDataDto.getDataScenarios());
        adminDataWithBLOBs.setFieldDescription(adminDataDto.getFieldDescription());
        adminDataWithBLOBs.setStatus(0); //默认下架，发布后上架
        adminDataWithBLOBs.setCreateTime(new Date());
        if (pictureFile != null) {
            String pictureUrl = fileUtil.upload(pictureFile);
            adminDataWithBLOBs.setPictureUrl(pictureUrl);
        }

        //设置数据文件
        if (dataFile != null && !dataFile.isEmpty()) {
            List<AdminFile> list = new ArrayList<>();
            for (MultipartFile file : dataFile) {
                AdminFile adminFile = new AdminFile();
                adminFile.setFileId(IdUtil.randomUUID());
                String dataUrl = fileUtil.upload(file);
                String fullFileName = file.getOriginalFilename();
                adminFile.setFileUrl(dataUrl);
                adminFile.setName(fullFileName);
                adminFile.setSource("数据表");
                adminFile.setDataCaseId(dataId);
                list.add(adminFile);
            }
            //批量新增数据文件
            adminFileMapper.insertBatch(list);
        }

        //批量新增标签
        List<AdminDataLabel> list = new ArrayList<>();
        for (AdminLabel adminLabel : adminLabelList) {
            AdminDataLabel label = new AdminDataLabel();
            label.setId(IdUtil.randomUUID());
            label.setLabelId(adminLabel.getLabelId());
            label.setName(adminLabel.getName());
            label.setDataCaseId(dataId);
            label.setType(adminLabel.getType());
            list.add(label);
        }
        adminDataLabelMapper.insertBatch(list);
        //新增案例
        adminDataMapper.insert(adminDataWithBLOBs);
        return new ResultEntity<>(HttpStatus.OK, "新增成功！");
    }

    @PostMapping("updateStatus")
    @ApiOperation("发布或禁用")
    @AnonymousAccess
    private ResultEntity<String> updateStatus(@RequestParam String id,
                                              @ApiParam("传0禁用，传1发布上架") @RequestParam int status) {
        try {
            AdminDataWithBLOBs adminDataWithBLOBs = adminDataMapper.selectByPrimaryKey(id);
            adminDataWithBLOBs.setStatus(status);
            adminDataMapper.updateByPrimaryKeyWithBLOBs(adminDataWithBLOBs);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "发生错误，请联系管理员！");
        }
        if (status == 0) {
            return new ResultEntity<>(HttpStatus.OK, "成功禁用！");
        }
        if (status == 1) {
            return new ResultEntity<>(HttpStatus.OK, "成功发布！");
        }
        return null;
    }


    @PostMapping("get")
    @ApiOperation("展示")
    @AnonymousAccess
    private ResultEntity<PageInfo<AdminDataReturnDto>> add(@RequestParam Integer index,
                                                           @RequestParam Integer size,
                                                           @RequestParam(required = false) String keyWord,
                                                           @ApiParam("精品案例不传，院校案例1") @RequestParam(required = false) String type,
                                                           @RequestParam(required = false) String labelName) {
        List<AdminDataReturnDto> adminDataReturnDtos = adminDataMapper.selectByConditions(keyWord, type, labelName);
        PageInfo pageInfo = PageUtil.pageHelper(adminDataReturnDtos, index, size);
        return new ResultEntity<>(pageInfo);
    }


    @PostMapping("updateAdminData")
    @ApiOperation("编辑")
    @AnonymousAccess
    @Transactional
    ResultEntity<String> updateAdminData(@RequestParam String adminDataDtoJSON,
                                         @RequestParam(required = false) MultipartFile pictureFile) {
        Gson gson = new Gson();
        AdminDataDto adminDataDto = null;
        try {
            adminDataDto = gson.fromJson(adminDataDtoJSON, AdminDataDto.class);
        } catch (JsonSyntaxException e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "服务器错误，请联系管理员");
        }
        //新增
        try {
            AdminDataWithBLOBs adminDataWithBLOBs = new AdminDataWithBLOBs();
            adminDataWithBLOBs.setDataId(adminDataDto.getDataId());
            adminDataWithBLOBs.setName(adminDataDto.getName());
            adminDataWithBLOBs.setDataScenarios(adminDataDto.getDataScenarios());
            adminDataWithBLOBs.setFieldDescription(adminDataDto.getFieldDescription());
            adminDataWithBLOBs.setStatus(adminDataDto.getStatus()); //默认下架，发布后上架
            adminDataWithBLOBs.setCreateTime(adminDataDto.getCreateTime());
            // 文件上传并行处理
            if (pictureFile != null) {
                String pictureUrl = fileUtil.upload(pictureFile);
                adminDataWithBLOBs.setPictureUrl(pictureUrl);
            }


            // 更新标签，批量更新操作
            if (adminDataDto.getAdminLabelList() != null && !adminDataDto.getAdminLabelList().isEmpty()) {
                List<AdminLabel> adminLabelList = adminDataDto.getAdminLabelList();
                AdminDataLabelExample example = new AdminDataLabelExample();
                example.createCriteria().andDataCaseIdEqualTo(adminDataDto.getDataId());
                adminDataLabelMapper.deleteByExample(example);

                //批量新增标签
                List<AdminDataLabel> list = new ArrayList<>();
                for (AdminLabel adminLabel : adminLabelList) {
                    AdminDataLabel label = new AdminDataLabel();
                    label.setId(IdUtil.randomUUID());
                    label.setLabelId(adminLabel.getLabelId());
                    label.setName(adminLabel.getName());
                    label.setDataCaseId(adminDataDto.getDataId());
                    label.setType(adminLabel.getType());
                    list.add(label);
                }
                // 使用批量更新操作
                if (!list.isEmpty()) {
                    adminDataLabelMapper.insertBatch(list);
                }
            }
            // 更新对象
            adminDataMapper.updateByPrimaryKeySelective(adminDataWithBLOBs);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "编辑失败请联系管理员！");
        }
        return new ResultEntity<>(HttpStatus.OK, "编辑成功！");
    }


    @PostMapping("addFile")
    @ApiOperation("编辑/文件单独新增")
    @AnonymousAccess
    private ResultEntity<HttpStatus> addFile(@RequestParam String dataId,
                                             @RequestParam MultipartFile dataFile) {
        try {
            AdminFile adminFile = new AdminFile();
            String url = fileUtil.upload(dataFile);
            adminFile.setFileId(IdUtil.randomUUID());
            adminFile.setDataCaseId(dataId);
            adminFile.setName(dataFile.getOriginalFilename());
            adminFile.setSource("管理员");
            adminFile.setFileUrl(url);
            adminFileMapper.insert(adminFile);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "上传失败，请联系管理员！");
        }
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "上传成功！");
    }


    @PostMapping("deleteFile")
    @ApiOperation("编辑/单个数据文件删除")
    @AnonymousAccess
    private ResultEntity<HttpStatus> deleteFile(@RequestParam String fileId) {
        try {
            AdminFile adminFile = adminFileMapper.selectByPrimaryKey(fileId);
            if (adminFile != null) {
                adminFileMapper.deleteByPrimaryKey(adminFile.getFileId());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "删除失败，请联系管理员！");
        }
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "删除成功！");
    }
}
