package com.sztzjy.resource_center.controller.new_module.stu;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.admin.AdminCaseReturnDto;
import com.sztzjy.resource_center.entity.admin.AdminCollectCase;
import com.sztzjy.resource_center.mapper.admin.AdminCaseMapper;
import com.sztzjy.resource_center.mapper.admin.AdminCollectCaseMapper;
import com.sztzjy.resource_center.mapper.admin.AdminFileMapper;
import com.sztzjy.resource_center.util.CompressUtil;
import com.sztzjy.resource_center.util.PageUtil;
import com.sztzjy.resource_center.util.ResultEntity;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Author xcj
 * @Date 2024/7/16
 */
@RestController
@Api(tags = "学生端案例相关")
@RequestMapping("api/stu/case")
public class StuCaseController {
    @Autowired
    private AdminCollectCaseMapper adminCollectCaseMapper;
    @Autowired
    private AdminCaseMapper adminCaseMapper;
    @Autowired
    private AdminFileMapper adminFileMapper;
    @Autowired
    private IFileUtil fileUtil;
    @Value("${file.path}")
    private String filePath;


    @PostMapping("/addCollectCase")
    @ApiOperation("收藏案例")
    @AnonymousAccess
    public ResultEntity<HttpStatus> download(@RequestParam String caseId,
                                             @RequestParam String userId) {
        try {
            AdminCollectCase adminCollectCase = new AdminCollectCase();
            adminCollectCase.setId(IdUtil.randomUUID());
            adminCollectCase.setCaseId(caseId);
            adminCollectCase.setUserId(userId);
            adminCollectCaseMapper.insert(adminCollectCase);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<HttpStatus>(HttpStatus.INTERNAL_SERVER_ERROR, "系统发生错误，请联系管理员!");
        }
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "收藏成功！");
    }


    @PostMapping("get")
    @ApiOperation("展示")
    @AnonymousAccess
    private ResultEntity<PageInfo<AdminCaseReturnDto>> get(@RequestParam Integer index,
                                                           @RequestParam Integer size,
                                                           @RequestParam(required = false) String keyWord,
                                                           @ApiParam("精品案例传管理员，院校案例传学校ID") @RequestParam(required = false) String source,
                                                           @ApiParam("收藏案例页面传") @RequestParam(required = false) String userId,
                                                           @RequestParam(required = false) String labelName) {
        List<String> list = new ArrayList<>();
        if (StringUtils.isNotBlank(labelName)) {
            String[] split = labelName.split(",");
            list = Arrays.asList(split);
        }
        List<AdminCaseReturnDto> adminCaseReturnDtos = adminCaseMapper.selectByConditionsByStu(keyWord, source, userId, list);
        PageInfo pageInfo = PageUtil.pageHelper(adminCaseReturnDtos, index, size);
        return new ResultEntity<>(pageInfo);
    }

    /**
     * 下载文件 支持多个下载
     * 格式 zip包
     *
     * @param ids
     */
    @PostMapping("/downloadZIP")
    @ApiOperation("数据文件打包下载")
    @AnonymousAccess
    public void download(@RequestBody List<String> ids, HttpServletResponse response) {
        this.downloadFile(ids, response);
    }


    public void downloadFile(List<String> ids, HttpServletResponse response) {
        if (ids != null && !ids.isEmpty()) {
            List<Map<String, String>> filePaths = adminFileMapper.getByIds(ids);
            String zipName = "TZ_" + ((int) (Math.random() * 10000)) + ".zip";
            String zipPath = filePath + "/" + zipName;
            CompressUtil.compress(filePaths, zipPath, false);
            File pocZipFile = new File(zipPath);
            CompressUtil.downloadZip(response, zipName, pocZipFile);
            pocZipFile.delete();
        }
    }

}
