package com.sztzjy.resource_center.controller.new_module.stu;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.admin.*;
import com.sztzjy.resource_center.mapper.admin.AdminCollectDataMapper;
import com.sztzjy.resource_center.mapper.admin.AdminDataMapper;
import com.sztzjy.resource_center.mapper.admin.AdminFileMapper;
import com.sztzjy.resource_center.util.PageUtil;
import com.sztzjy.resource_center.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Author xcj
 * @Date 2024/7/17
 */
@RestController
@Api(tags = "学生端数据相关")
@RequestMapping("api/stu/data")
public class StuDataController {
    @Autowired
    private AdminCollectDataMapper adminCollectDataMapper;
    @Autowired
    private AdminDataMapper adminDataMapper;
    @Autowired
    private AdminFileMapper adminFileMapper;

    @PostMapping("/addCollectData")
    @ApiOperation("添加收藏案例")
    @AnonymousAccess
    public ResultEntity<String> download(@RequestParam String dataId,
                                         @RequestParam String userId) {
        try {
            AdminCollectData adminCollectData = new AdminCollectData();
            adminCollectData.setId(IdUtil.randomUUID());
            adminCollectData.setDataId(dataId);
            adminCollectData.setUserId(userId);
            adminCollectDataMapper.insert(adminCollectData);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "系统发生错误，请联系管理员");
        }
        return new ResultEntity<>(HttpStatus.OK, "收藏成功！");
    }


    @PostMapping("getById")
    @ApiOperation("展示单个详情")
    @AnonymousAccess
    private ResultEntity<AdminDataReturnDto> getById(@RequestParam String id) {
        AdminDataWithBLOBs adminDataWithBLOBs = adminDataMapper.selectByPrimaryKey(id);
        AdminDataReturnDto adminDataReturnDto = new AdminDataReturnDto();
        BeanUtils.copyProperties(adminDataWithBLOBs, adminDataReturnDto);
        AdminFileExample example = new AdminFileExample();
        example.createCriteria().andDataCaseIdEqualTo(id);
        List<AdminFile> adminFiles = adminFileMapper.selectByExample(example);
        adminDataReturnDto.setAdminFiles(adminFiles);
        return new ResultEntity<AdminDataReturnDto>(adminDataReturnDto);
    }

    @PostMapping("selectByConditions")
    @ApiOperation("数据仓库展示")
    @AnonymousAccess
    private ResultEntity<PageInfo<AdminDataReturnDto>> selectByConditions(@RequestParam Integer index,
                                                                          @RequestParam Integer size,
                                                                          @RequestParam(required = false) String keyWord,
                                                                          @ApiParam("为空展示所有数据，院校数据传学校ID") @RequestParam(required = false) String source,
                                                                          @ApiParam("收藏案例页面传") @RequestParam(required = false) String userId,
                                                                          @RequestParam(required = false) String labelName) {
       List<AdminDataReturnDto> adminDataReturnDtos = adminDataMapper.selectByConditionsBySchoolId(keyWord, source, userId, labelName);
        PageInfo pageInfo = PageUtil.pageHelper(adminDataReturnDtos, index, size);
        return new ResultEntity<PageInfo<AdminDataReturnDto>>(pageInfo);
    }
}
